# Contenu
1. [Initialisation du système](#s1)
2. [Protected Mode](#s2)
3. [Structure du projet](#s3)

# Initialisation du système<div id='s1'/>
- le BIOS lance en [Real Mode](https://wiki.osdev.org/Real_Mode) (16 bits)
- il faut passer en [Protected Mode](https://wiki.osdev.org/Protected_Mode) (32 bits) : pour l'instant, GRUB s'en charge pour nous
- si un jour on décide de coder un OS qui tourne avec des instructions 64 bits : passer en [Long Mode](https://wiki.osdev.org/Long_Mode), mais pas nécessaire pour l'instant

# Protected Mode<div id='s2'/>
À implémenter, par ordre de priorité :
- [Global Descriptor Table](https://wiki.osdev.org/GDT) ([tutoriel](https://wiki.osdev.org/GDT_Tutorial)), pour indiquer la segmentation de la mémoire en Protected Mode. On peut se contenter d'un modèle plat minimal, pour l'instant et peut-être même pour tout le temps (le nombre minimal de segments, qui couvrent tous toute la mémoire). **OK**
- [Interrupt Descriptor Table](https://wiki.osdev.org/Interrupt_Descriptor_Table), pour relier les interrupt aux actions à effectuer. **partiellement OK**
- Peut-être rajouter un clavier à ce moment là
- Mémoire virtuelle (paging)

## GDT, segmentation
On a le choix entre deux modèles pour la mémoire : [segmentation](https://wiki.osdev.org/Segmentation) ou [paging](https://wiki.osdev.org/Paging). De ce que je comprends, quasiment tout le monde utilise le paging désormais et la segmentation est surtout un vestige d'architectures + anciennes, pour des raisons de rétrocompatibilité (insérer emoji clown ici). Donc il n'y aura pas besoin de mettre trop de trucs dedans, juste le minimum syndical pour faire tourner le protected mode et passer à la suite.

On va décrire une segmentation de notre mémoire avec une table de descripteurs que le CPU sait lire (donc avec un format assez délirant, merci Intel ; ce format est décrit dans le fichier `gdt.h`).
En outre, on fournit un descripteur du GDT, qui contient sa taille (sur 16 bits) et son adresse (sur 32 bits). On peut alors charger le GDT avec l'instruction `lgdt`, en passant la valeur stockée à l'adresse du descripteur. Après, il faut aussi mettre à jour les registres de segment :
- pour le registre du segment de code `%cs`, il suffit de faire un far jump avec l'instruction `ljump` vers un label en précisant le segment de code voulu.
- pour les autres (`%ds` le segment de données, `%ss` le segment de pile et `%es`, `%fs` et `%gs` les registres supplémentaires), on les initialise tous pour l'instant au segment de données (si on fait le modèle plat).

Des explications complémentaires sont dans les commentaires des fichiers du GDT.

## Interrupts
### IDT
Encore une fois, c'est juste une grande table avec des descripteurs avec un format légèrement moins horrible que pour le GDT. Lorsque l'on reçoit un interrupt $i$, le processeur se charge d'aller regarder la $i$-ème case de l'IDT pour savoir quoi faire.

Chaque case contient un descripteur d'une porte (gate), c'est à dire d'une action à faire lorsque l'interrupt correspondant est reçu. On distingue 3 types de portes :
- Trap gate : à utiliser pour la gestion des [exceptions](https://wiki.osdev.org/Exceptions) (émises par le processeur).
- Interrupt gate : à utiliser pour la gestion des [IRQ](https://wiki.osdev.org/IRQ) (Interrupt Request). La seule différence avec une trap gate est que dans ce cas, les interrupt sont désactivés pendant la gestion de l'IRQ, ce qui n'est pas le cas avec les exceptions.
- Task gate : sert lorsque l'on veut faire un Hardware Task Switching, c'est à dire jamais pour nous, très probablement.

Une description plus détaillée du format des descripteurs est donnée dans l'entête du fichier `idt.h`.

### Exceptions
Ce sont des interrupt internes utilisés par le processeur. Ils sont associés aux entrées 0 à 31 de l'IDT, et peuvent être de trois types :
- Fault : une erreur mineure qui peut éventuellement être corrigée. En sortant de l'[ISR](https://wiki.osdev.org/Interrupt_Service_Routines) (Interrupt Service Routine, c'est à dire la fonction pointée par la case de l'IDT correspondant à l'instruction), l'adresse de retour est _celle de l'instruction fautive_.
- Trap : une interruption qui n'est pas due à une erreur (debug, breakpoint, overflow). L'adresse de retour à la sortie de l'ISR est celle de l'instruction suivante.
- Abort : une erreur irrécupérable qui nécessite de tuer le processus.

En outre, certaines exceptions fournissent un code d'erreur, qui est mis sur la pile (et donc utilisable comme argument de l'ISR).

Le seul cas particulier est l'interrupt 2 ([NMI](https://wiki.osdev.org/NMI), Non Maskable Interrupt), qui ne doit pas être géré avec une Trap Gate mais avec une Interrupt Gate.

La liste de ces exceptions et leur description est donnée sur la [page correspondante](https://wiki.osdev.org/Exceptions), ou dans le manuel Intel.

### PIC
Le [PIC](https://wiki.osdev.org/PIC) est une puce qui se charge de récupérer les interrupt envoyés par le matériel, et de les associer aux interrupts voulus dans l'IDT.

# Structure du projet<div id='s3'/>
## `kernel`
Contient le code du noyau. Le dossier `arch` contient, pour chaque architecture supportée (c'est à dire uniquement i386), la programmation des éléments spécifiques à cette architecture : structures de données, hardware, interrupt... Sert à créer une couche d'abstraction.

Faire un µ-noyau ?
