#include "vga.h"

#include <kernel/tty.h>

#include <string.h>

static const size_t VGA_WIDTH = 80;
static const size_t VGA_HEIGHT = 25;
static const size_t DISPLAY_ADDRESS = 0xB8000;

size_t term_row;
size_t term_column;
display_color term_color;
display *term_buffer;
size_t split_line;

//changer de couleur
void term_setcolor(display_color color) 
{
	term_color = color;
}

//changer/récupérer une entrée VGA
void term_setentryat(char c, display_color color, size_t row, size_t column)
{
	const size_t ind = row * VGA_WIDTH + column;
	term_buffer[ind] = vga_entry(c, color);
}

display term_getentryat(size_t row, size_t column)
{
	const size_t ind = row * VGA_WIDTH + column;
	return term_buffer[ind];
}

//effacer le terminal
void term_clearat(size_t row, size_t column)
{
	term_setentryat(' ', term_color, row, column);
}

void term_clearrow(size_t row)
{
	for (size_t column = 0; column < VGA_WIDTH; column++)
		term_clearat(row, column);
}

void term_clear()
{
	for (size_t row = 0; row < VGA_HEIGHT; row++)
		term_clearrow(row);
}

//défilement d'une ligne
void term_scroll()
{
	for (size_t row = 1; row < VGA_HEIGHT; row++)
		for (size_t column = 0; column < VGA_WIDTH; column++)
			term_setentryat(term_getentryat(row, column), term_color, row - 1, column);
}

//Changement de lignes
void term_newline()
{
	++term_row;
	term_column = 0;
	if (term_row == VGA_HEIGHT)
	{
		term_scroll();
		term_clearrow(--term_row);
	}
}

//initialisation
void term_init()
{
	term_row = 0;
	term_column = 0;
	term_setcolor(vga_entry_color(VGA_WHITE, VGA_BLACK));
	term_buffer = (display *)DISPLAY_ADDRESS;
	split_line = 0;
	term_clear();

	term_writestring("Terminal initialized.\n");
}

//affichage
void term_putchar(char c)
{
	switch (c)
	{
	case '\n':
		term_newline();
		break;
	case '\b':
		if (term_column > 0)
			term_column--;
		else if (split_line > 0)
		{
			term_row--;
			term_column = VGA_WIDTH - 1;
			split_line--;
		}
		term_setentryat(' ', term_color, term_row, term_column);
		break;
	default:
		term_setentryat(c, term_color, term_row, term_column);
		if (++term_column == VGA_WIDTH)
		{
			term_newline();
			split_line++;
		}
	}
}

void term_write(const char *data, size_t size)
{
	for (size_t i = 0; i < size; i++)
		term_putchar(data[i]);
}

void term_writestring(const char *str)
{
	term_write(str, strlen(str));
}
