#include "pit.h"

#include <kernel/interrupt.h>
#include <kernel/portio.h>
#include <kernel/tty.h>

#include <stddef.h>

uint8_t programming = 0;
uint8_t cnt;

//-- ISR --
__attribute__((interrupt))
void pit_isr(interrupt_frame *frame __attribute__((unused)))
{
	term_writestring("tick!");
	send_EOI(0);
}

//-- Programmer un seul canal --
void pit_program_ch(pit_channel ch, pit_access acc, pit_mode mod, uint16_t count)
{
	outb(PIT_CMD_REG, ch << 6 | acc << 4 | mod << 1);
	if (acc & 0b1)
		outb(PIT_OUT + ch, count & 0xFF);
	if (acc & 0b10)
		outb(PIT_OUT + ch, count >> 0xFF);
}

//-- Fonctions du driver --
void pit_driver_init()
{
	pit_program_ch(PIT_IRQ, PIT_ACC_WHOLE, PIT_REPEAT, 0);
	set_interrupt(PIT_INT, &pit_isr);
	term_writestring("PIT driver initialized.\n");
}

//-- Driver --
timer_driver pit_driver = {
	{
		"PIT",
		&pit_driver_init,
		NULL,
		NULL
	}
};
